#!/usr/bin/perl
=pod

=head1 NAME

find-undeclared-subs.pl - find places in perl code where undeclared symbols
are used

=head1 SYNOPSIS

B<find-undeclared-subs.pl> S<[ B<-t> I<types-of-symbols-to-check> ]>
	S<[ B<-P> I<perl-backend-command> ]> I<file> ...


=head1 DESCRIPTION

This script lists all undeclared subs used in the perl source files 
passed on commandline. The use of this script gives more than plain placing
of C<'use strict'> in your source since in 
case of C<'use strict'> the use of undeclared subroutines is detected only at 
runtime when perl has to execute them (i.e. if execution path reached the 
invokation of the undeclared sub). 
In contrast this script examines all execution paths and reports all locations
the undeclared subroutines are used.

It does not check the modules that the analyzed file requires if the modules
are located not in the subdirectories of the directory where the analyzed
file is located.

=head1 OPTIONS

=over 4

=item B<-t> I<types-of-symbols-to-check>

Specify use of what types of undeclared symbols you wish to list. 
The I<types-of-symbols-to-check> is a string of I<symbol prefixes> you
wish to list, listed in any order without any space between them.
Here is a correspondence between I<symbol prefixes> and types of symbols: 

=over 4

=item B<&>

Subroutine

=item B<$>

Scalar variable

=item B<@>

List variable

=item B<%>

Hash variable

=item B<*>

Symbol table entry

=back

The default value of this argument is B<&> - i.e. the locations of the use
of undeclared subroutines is listed.

=item B<-P> I<perl-backend-command>

Specify the invokaction string for the backend. It does not need to specify
the full path of the perl program. This option is intended for passing any 
additional options mostly used for specifying the location of modules your
program requires. The passed string is prepended to the necessary options
used for functionality of this script. The default value of this option is
I<perl>.

=back

=head1 NOTES

It can list give false alarms for symbols that are not subroutines
(this is a feature of the B<B::Xref> module used as a backend).
So it is only practical to check only the places with the use of undeclared
subroutines (that's the default mode if you don't pass any B<-t> options).
Use the C<'use strict'> in your source for detecting use of undeclared 
objects of types other than subroutines.



=head1 EXAMPLES

The following commandline should list all locations where undeclared hashes
and subroutines are used in files F<file1.pl> and F<file2.pl>, 
instructing to search for modules in the parent directory:

 find-undeclared-subs.pl -t '%&' -P 'perl -I..' file1.pl file2.pl


=cut
die('this is a stub of find-undeclared-subs.pl. To evaluate a product, ask Stunnix support for a real trial via email');
